extern crate yaml_rust;

mod builder;
mod theme;
mod preview;

use builder::{clean, download_sources};
use clap::{Parser, Subcommand};
use directories::BaseDirs;
use theme::{apply, list};
use preview::{preview};

// TODO: When accessing elements in the source.toml file, it will panic if it doesn't exist
// TODO: Make error parsing / checking better overall: use ?, return Results and parse them in the
// end and have better error output instead of panic

#[derive(Parser, Debug)]
struct Args {
    #[clap(subcommand)]
    command: Commands,
}

#[derive(Subcommand, Debug)]
enum Commands {
    /// Lists all themes
    List {
        /// Name of source to list themes from
        #[clap(short, long)]
        source: Option<String>,
    },
    /// List all templates
    Template {
        #[clap(short, long)]
        source: Option<String>,
    },
    /// Applies themes
    Apply {
        /// Builds files from config and applies a theme of your choice
        theme: Option<String>,
    },
    /// Update sources
    Update {},
    /// Cleans source directories
    Clean {},
    /// Preview Colors
    Preview {
        /// Preview specific theme
        #[clap(short, long)]
        theme: Option<String>,
    }
}

fn main() {
    let options = Args::parse();

    let base_dirs = BaseDirs::new().expect("Could not find configuration folder."); // Shouldn't happen?

    let data_path = base_dirs.data_dir().join("themectl");

    let config = base_dirs.config_dir().join("themectl/config.toml");

    let sources = base_dirs.config_dir().join("themectl/sources.toml");
    let cache = base_dirs.cache_dir().join("themectl");

    match &options.command {
        Commands::Clean {} => {
            clean(&data_path).unwrap();
        }
        Commands::Update {} => {
            download_sources(&sources, &data_path, &cache).unwrap();
        }
        Commands::Apply { theme } => {
            apply(&config, &data_path, &sources, theme).unwrap();
        }
        Commands::List { source } => {
            list(&sources, &data_path, source, "schemes").unwrap();
        }
        Commands::Template { source } => {
            list(&sources, &data_path, source, "templates").unwrap();
        }
        Commands::Preview { theme } => {
            preview(&data_path, &sources, theme.clone()).unwrap();
        }
    }
}
