use std::process::Command;
use ansi_term::Color::RGB;
use std::collections::HashMap;
use crate::builder::yaml_from_file;
use std::path::Path;
use std::io;
use std::io::{Read};
use toml::{Value, map::Map};
use std::fs;

pub fn preview(store: &Path, data: &Path, theme: Option<String>) -> Result<(), io::Error> { 
    if Command::new("clear").status().is_ok() {};

    let mut stdin = io::stdin();

    let path = match theme {
        Some(x) => { 
            let mut schemes: HashMap<String, String> = HashMap::new();

            let data = fs::read_to_string(data)?
                .parse::<Value>()?;

            for local in data
                .get("schemes")
                .unwrap_or(&Value::Table(Map::new()))
                .get("local")
                .unwrap_or(&Value::Array([].to_vec()))
                .as_array()
                .ok_or_else(|| io::Error::new(io::ErrorKind::InvalidData, "Incorrect sources format"))?
                .iter()
            {
                let dict = fs::read_dir(Path::new(
                    &local.as_str().unwrap().replace("~", env!("HOME")),
                ))?
                .map(|x| String::from(x.unwrap().path().to_str().unwrap()))
                .collect::<Vec<String>>();

                for i in dict {
                    schemes.insert(Path::new(&i).file_stem().unwrap().to_str().unwrap().to_string(),
                                     i.clone());
                }

            }

            let dict = fs::read_dir(&store.join("schemes"))?
                .map(|x| String::from(x.unwrap().path().to_str().unwrap())) // This shouldn't error
                .collect::<Vec<String>>();


            for i in dict {
                schemes.insert(Path::new(&i).file_stem().unwrap().to_str().unwrap().to_string(),
                                 i.clone());
            }


            store.join("schemes").join(schemes.get(&x)
                                       .ok_or_else(|| io::Error::new(io::ErrorKind::InvalidInput, "Invalid theme"))?
                                       )
            }
        None => store.join("current.yaml")
    };
    let yaml = &yaml_from_file(&path)?[0];

    let mut colors: HashMap<i8, [u8; 3]> = HashMap::new();

    for (a, v) in yaml
        .as_hash()
        .ok_or_else(|| io::Error::new(io::ErrorKind::InvalidData, "Error reading theme file."))?
        .iter() {

            let val = v.as_str()
                .ok_or_else(|| io::Error::new(io::ErrorKind::InvalidData, "Error reading theme file."))?
                .to_string();

            match a.as_str().unwrap() {
                "scheme" => (),
                "author" => (),

                _ => {
                    colors.insert(
                        i8::from_str_radix(
                            &a.as_str().unwrap()[4..], 16)
                        .unwrap()
                        , [
                        u8::from_str_radix(&val[0..2].to_string(), 16).unwrap(),
                        u8::from_str_radix(&val[2..4].to_string(), 16).unwrap(),
                        u8::from_str_radix(&val[4..6].to_string(), 16).unwrap(),
                        ]);
                }
            }
    }


    println!();
    for i in 0..8 {
        print!("{}", 
             RGB(colors[&i][0], colors[&i][1], colors[&i][2]).on(RGB(colors[&i][0], colors[&i][1], colors[&i][2])).paint("    "));
    }
    println!();

    for i in 0..8 {
        print!("{}", 
             RGB(colors[&i][0], colors[&i][1], colors[&i][2]).on(RGB(colors[&i][0], colors[&i][1], colors[&i][2])).paint("    "));
    }
    println!();

    for i in 8..16 {
        print!("{}", 
             RGB(colors[&i][0], colors[&i][1], colors[&i][2]).on(RGB(colors[&i][0], colors[&i][1], colors[&i][2])).paint("    "));
    }
    println!();

    for i in 8..16 {
        print!("{}", 
             RGB(colors[&i][0], colors[&i][1], colors[&i][2]).on(RGB(colors[&i][0], colors[&i][1], colors[&i][2])).paint("    "));
    }
    println!();
    println!();

    stdin.read_exact(&mut [0u8]).unwrap();

    Ok(())
}
